@@ -1,18 +1,50 @@
+# Copyright (C) <2021>  YUANXIN INFORMATION TECHNOLOGY GROUP CO.LTD and Jinzhe Wang
+# This file is part of uitestrunner_syberos
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+import multiprocessing
+
+import Item
 import base64
 import json
 import os
 import re
 import threading
+import time
 
-from . import Connection
-from . import Application
-from . import Events
+from .Connection import Connection
+from .Application import Application
+from .Events import Events
 from urllib3 import encode_multipart_formdata
 import configparser
+from multiprocessing import Process, Pipe
+from .Watcher import *
 
 
-class Device(Events.Events):
-    con = Connection.Connection()
+def _watcher_process(main_pid, host, port, conn):
+    device = Device(host=host, port=port, main=False)
+    WatchWorker(device, conn, main_pid).run()
+
+
+def _start_watcher(host, port, watcher_conn):
+    watcher_process = Process(target=_watcher_process, args=(os.getpid(), host, port, watcher_conn))
+    watcher_process.daemon = True
+    watcher_process.start()
+
+
+class Device(Events):
+    con = Connection()
     __osVersion = ""
     __serialNumber = ""
     xmlStr = ""
@@ -25,21 +57,37 @@ class Device(Events.Events):
     __syslog_save_path = "./syslog/"
     __syslog_save_name = ""
     __syslog_save_keyword = ""
+    __runner_log = True
+    watcher_list = []
+    __main_conn, __watcher_conn = Pipe()
 
-    def __init__(self, host="192.168.100.100", port=8080):
+    def __init__(self, host="192.168.100.100", port=8080, runner_log=True, main=True):
         super().__init__(d=self)
+        self.__runner_log = runner_log
         self.con.host = host
         self.con.port = port
         self.con.connect()
         self.__serialNumber = str(self.con.get(path="getSerialNumber").read(), 'utf-8')
         self.__osVersion = str(self.con.get(path="getOsVersion").read(), 'utf-8')
-        self.syslog_thread = threading.Thread(target=self.__logger)
-        self.syslog_thread.setDaemon(True)
-        self.syslog_thread.start()
+        if main:
+            _start_watcher(host, port, self.__watcher_conn)
+            syslog_thread = threading.Thread(target=self.__logger)
+            syslog_thread.setDaemon(True)
+            syslog_thread.start()
+        self.refresh_layout()
+        self.log("debug", "Device, create Device object, host=%s, port=%s" % (host, str(port)))
+
+    def push_watcher_data(self):
+        self.__main_conn.send({'watcher_list': self.watcher_list})
+
+    def watcher(self, name: str):
+        w = Watcher({'name': name}, self)
+        return w
 
     def __logger(self):
         syslog_save_path = ""
         syslog_file = None
+        self.log("debug", "__logger(), logger started.")
         messages = self.device.con.sse("SysLogger")
         for msg in messages:
             log_str = str(msg.data)
@@ -57,8 +105,10 @@ class Device(Events.Events):
     def set_syslog_output(self, is_enable, keyword=""):
         self.__syslog_output_keyword = keyword
         self.__syslog_output = is_enable
+        self.log("debug", "set_syslog_output(), is_enable=%s, keyword=\"%s\"" % (str(is_enable), keyword))
 
     def syslog_output(self):
+        self.log("debug", "syslog_output(), is_enable=%s" % (str(self.__syslog_output)))
         return self.__syslog_output
 
     def syslog_output_keyword(self):
@@ -107,16 +157,16 @@ class Device(Events.Events):
         image.close()
         return file_name
 
-    def get_xpath(self, sopid, key):
+    def get_xpath(self, sop_id, key):
         if not os.path.exists(self.__xpath_file):
             f = open(self.__xpath_file, "w")
             f.close()
         conf = configparser.ConfigParser()
         conf.read(self.__xpath_file)
-        return conf.get(sopid, key)
+        return conf.get(sop_id, key)
 
-    def application(self, sopid, uiappid):
-        return Application.Application(d=self, sop=sopid, ui=uiappid)
+    def application(self, sop_id, ui_app_id):
+        return Application(d=self, sop=sop_id, ui=ui_app_id)
 
     def refresh_layout(self):
         self.xmlStr = str(self.con.get(path="getLayoutXML").read(), 'utf-8')
@@ -167,3 +217,12 @@ class Device(Events.Events):
 
     def uninstall(self, sopid):
         return bool(int(str(self.con.get(path="uninstall", args="sopid=" + sopid).read(), 'utf-8')))
+
+    def log(self, level, text):
+        pass
+        # if not self.__runner_log:
+        #     return
+        # local_time = time.strftime("%Y%m%d-%H:%M:%S")
+        # remote_time_raw_str = str(self.con.get(path="getSystemTime").read(), 'utf-8')[:-3]
+        # remote_time = "|R" + remote_time_raw_str[0:8] + "-" + remote_time_raw_str[8:10] + ":" + remote_time_raw_str[10:12] + ":" + remote_time_raw_str[12:14]
+        # print("[L%s%s]:%s:%s" % (local_time, remote_time, level, text))
